@@ -106,7 +106,7 @@ class SubprocessFilter(Filter):
             'scriptargs' : self.command_line_scriptargs() or "",
             'script_file' : self.input_filename()
         }
-        return "%(prog)s %(args)s %(script_file)s %(scriptargs)s" % args
+        return """%(prog)s %(args)s "%(script_file)s" %(scriptargs)s""" % args
 
     def command_string(self):
         args = {
@@ -116,7 +116,7 @@ class SubprocessFilter(Filter):
             'scriptargs' : self.command_line_scriptargs() or "",
             'output_file' : self.output_filename()
         }
-        return "%(prog)s %(args)s %(script_file)s %(scriptargs)s %(output_file)s" % args
+        return """%(prog)s %(args)s "%(script_file)s" %(scriptargs)s "%(output_file)s" """ % args
 
     def ignore_nonzero_exit(self):
         return self.artifact.wrapper.ignore_nonzero_exit
@@ -155,11 +155,11 @@ class SubprocessFilter(Filter):
         env.update(self.args().get('env', {}))
 
         # Add parameters in wrapper's env dict
-        for doc in self.processed():
-            if doc.final_artifact.ext == '.json':
-                for env_item, env_value in doc.output().json_as_dict().iteritems():
-                    if env_item.startswith("DEXY_"):
-                        env[env_item] = env_value
+        if self.is_part_of_script_bundle():
+            for key, value in self.script_storage().iteritems():
+                if key.startswith("DEXY_"):
+                    self.log.debug("Adding %s to env value is %s" % (key, value))
+                    env[key] = value
 
         # Add any path extensions to PATH
         if self.PATH_EXTENSIONS:
@@ -320,7 +320,8 @@ class SubprocessCompileFilter(SubprocessFilter):
     def compile_command_string(self):
         wf = os.path.basename(self.input().name)
         of = self.compiled_filename()
-        return "%s %s -o %s" % (self.executable(), wf, of)
+        compiler_args = self.args().get("compiler-args", "")
+        return "%s %s %s -o %s" % (self.executable(), compiler_args, wf, of)
 
     def compiled_filename(self):
         basename = os.path.basename(self.input().name)
@@ -365,7 +366,7 @@ class SubprocessInputFilter(SubprocessFilter):
     def process(self):
         command = self.command_string()
 
-        inputs = self.artifact.doc.completed_child_docs()
+        inputs = list(self.artifact.doc.node.walk_input_docs())
 
         output = OrderedDict()
 
@@ -378,13 +379,38 @@ class SubprocessInputFilter(SubprocessFilter):
                 output[section_name] = stdout
         else:
             for doc in inputs:
-                proc, stdout = self.run_command(command, self.setup_env(), doc.output().as_text())
+                proc, stdout = self.run_command(command, self.setup_env(), unicode(doc.output()))
                 if self.CHECK_RETURN_CODE:
                     self.handle_subprocess_proc_return(command, proc.returncode, stdout)
                 output[doc.key] = stdout
 
         self.output().set_data(output)
 
+class SubprocessInputFileFilter(SubprocessFilter):
+    """
+    Filters which run one or more input files through the script via filenames.
+    """
+    CHECK_RETURN_CODE = False
+    WRITE_STDERR_TO_STDOUT = False
+    OUTPUT_DATA_TYPE = 'sectioned'
+
+    def command_string_for_input(self, input_doc):
+        return "%s %s %s" % (self.executable(), self.input_filename(), input_doc.output().name)
+
+    def process(self):
+        inputs = list(self.artifact.doc.node.walk_input_docs())
+
+        output = OrderedDict()
+
+        for doc in inputs:
+            command = self.command_string_for_input(doc)
+            proc, stdout = self.run_command(command, self.setup_env())
+            if self.CHECK_RETURN_CODE:
+                self.handle_subprocess_proc_return(command, proc.returncode, stdout)
+            output[doc.key] = stdout
+
+        self.output().set_data(output)
+
 class SubprocessCompileInputFilter(SubprocessCompileFilter):
     """
     Filters which compile code, then run it with input.
@@ -401,7 +427,7 @@ class SubprocessCompileInputFilter(SubprocessCompileFilter):
 
         command = self.run_command_string()
 
-        inputs = self.artifact.doc.completed_child_docs()
+        inputs = list(self.artifact.doc.node.walk_input_docs())
 
         output = OrderedDict()
 
